

functions { 
  real gpcm(int y,  real theta, vector beta, vector covariates, vector gamma) {
    vector[rows(beta)+1] unsummed; 
    vector[rows(beta)+1] probs;
    vector[rows(beta)+1] cumsum;
    vector[rows(beta)] covariate_score_vector;
    vector[rows(beta)+1] covariate_score_vector_aug;
    real covariate_score = 0;
    for (i in 1:rows(gamma))
      covariate_score = covariate_score + covariates[i]*gamma[i]; 
       for (i in 1:rows(beta))
          covariate_score_vector[i] = covariate_score;
        
        covariate_score_vector_aug = append_row(rep_vector(0.0, 1), covariate_score_vector);
        unsummed = append_row(theta, (theta - beta));
        cumsum = cumulative_sum(unsummed);
        probs = softmax(cumsum+covariate_score_vector_aug);
        return categorical_lpmf(y + 1 | probs); 
      }  
    
    real gpcm_rng( real theta, vector beta, vector covariates, vector gamma) {
      vector[rows(beta)+1] unsummed;
      vector[rows(beta)+1] probs;
      vector[rows(beta)+1] cumsum;
      vector[rows(beta)] covariate_score_vector;
      vector[rows(beta)+1] covariate_score_vector_aug;
      real covariate_score = 0;
      for (i in 1:rows(gamma))
        covariate_score = covariate_score + covariates[i]*gamma[i];
      
      for (i in 1:rows(beta))
        covariate_score_vector[i] = covariate_score;
      
      covariate_score_vector_aug = append_row(rep_vector(0.0, 1), covariate_score_vector);
      unsummed = append_row(theta, (theta - beta));
      cumsum = cumulative_sum(unsummed);
      probs = softmax(cumsum+covariate_score_vector_aug);
      return categorical_rng( probs);
    }  
    
  }
  
  
  
  
  
  data {
    int<lower=1> J;                // # items
    int<lower=1> n;                // # persons
    int<lower=1> N;                // # total responses (n*J)
    int<lower=0> N_mis;                // # missing data points
    int<lower=0> m;                // # covariates
    int<lower=1,upper=J> jj[N];    // item j for i in N
    int<lower=1,upper=n> nn[N];    // person n for i in N
    int<lower=1,upper=J> jj_mis[N_mis];    // item j for i in N_mis
    int<lower=1,upper=n> nn_mis[N_mis];    // person n for i in N_mis
    int<lower=1> K[J];             // # categories item j -1 (non-base categories only)
    vector[n*m] covariates;
    int<lower=0> y[N];             // response for n; y = 0,1,2, ..., K_j  
  }
  
  transformed data {
    int pos_beta[J];   // used to segment the beta vector, get betas for each iten.
    int pos_cov[n];    // used to segment the covariate vector, get covariates for each individual.
    pos_beta[1] = 1;
    pos_cov[1] = 1;
    
    for(i in 2:J) pos_beta[i] = K[i-1] + pos_beta[i-1];
    for(i in 2:n) pos_cov[i] = m + pos_cov[i-1];
    
    
    
}
  
  parameters {
    vector[sum(K)] beta; //we only have a beta for non-base categories
    vector[n] theta;
    vector[m] gamma;
    
  }
 
  
  model {
    
    target += normal_lpdf(beta | 0, sqrt(10));
    theta ~ normal(0, 1);
    for(i in 1:m)
      gamma[i] ~ normal(0,sqrt(10000)); 
    
    for (i in 1:N)
      target += gpcm(y[i],  theta[nn[i]],// person/response is the ordering of data
                     segment(beta, pos_beta[jj[i]], K[jj[i]]), 
                     segment(covariates, pos_cov[nn[i]] , m), 
                     gamma);
      
  }
  
  generated quantities {
    vector[N_mis] y_mis;
    vector[N] log_lik;
    
    for (i in 1:N_mis)
      y_mis[i] = gpcm_rng( theta[nn_mis[i]],// person/response is the ordering of data
                          segment(beta, pos_beta[jj_mis[i]], K[jj_mis[i]]), 
                          segment(covariates, pos_cov[nn_mis[i]] , m), 
                          gamma);
    
    for (i in 1:N)
      log_lik[i] = gpcm(y[i], theta[nn[i]],// person/response is the ordering of data
                        segment(beta, pos_beta[jj[i]], K[jj[i]]), 
                        segment(covariates, pos_cov[nn[i]] , m), 
                        gamma);
    
  }          
                                                                                     
                                                                                     
  
  

  
  
  